import { toRelativePath, includeUrl, parse } from '../url';


describe('get-env', () => {
    test('toRelativePath', () => {
        expect(toRelativePath('https://www.gaoding.com')).toEqual('https://www.gaoding.com');
        expect(toRelativePath('https://www.gaoding.com', 'www.gaoding.com')).toEqual('/');
        expect(toRelativePath('https://www.gaoding.com', 'https://www.gaoding.com')).toEqual('/');
        expect(toRelativePath('https://www.gaoding.com/test', 'https://www.gaoding.com')).toEqual('/test');
        expect(toRelativePath('https://www.gaoding.com/test?a=1', 'https://www.gaoding.com')).toEqual('/test?a=1');
        expect(toRelativePath('https://www.gaoding.com/test?a=1&b=3', 'https://www.gaoding.com')).toEqual('/test?a=1&b=3');
        expect(toRelativePath('https://dianshang.gaoding.com/test?a=1&b=3', 'https://www.gaoding.com')).toEqual('https://dianshang.gaoding.com/test?a=1&b=3');
        expect(toRelativePath('www.gaoding.com', 'https://www.gaoding.com')).toEqual('/');
        expect(toRelativePath('www.gaoding.com/', 'https://www.gaoding.com')).toEqual('/');
    });

    test('includeUrl', () => {
        expect(includeUrl('https://www.gaoding.com', 'https://www.gaoding.com')).toEqual(true);
        expect(includeUrl('https://www.gaoding.com/test', 'https://www.gaoding.com')).toEqual(false);
        expect(includeUrl('https://www.gaoding.com/test/test2/?a=2', 'https://www.gaoding.com/test/test2')).toEqual(true);
    });

    test('parse', () => {
        expect(parse('https://www.gaoding.com')).toEqual({ hostname: 'www.gaoding.com', params: {}, pathname: '/' });
        expect(parse('www.gaoding.com/test/test2?ab=1')).toEqual({ hostname: 'www.gaoding.com', pathname: '/test/test2', params: { ab: '1' } });
    });
});
