const cacheName = 'gaodingx-cache';
const VERSION = '1.0.0';
function getTime(time) {
    if (typeof time === 'number')
        { return time; }
    if (typeof time === 'string' && /([.\d]+)([DdSsHhMm])$/.test(time)) {
        const value = +RegExp.$1;
        switch (RegExp.$2.toLocaleLowerCase()) {
            case 'd':
                return value * 24 * 60 * 60 * 1000;
            case 'h':
                return value * 60 * 60 * 1000;
            case 'm':
                return value * 60 * 1000;
            case 's':
                return value * 1000;
            default:
                break;
        }
    }
    throw new Error(`无效的时间${time}`);
}
function createCache(api) {
    return {
        get(key, options = { version: VERSION, time: '7d' }) {
            const { version = VERSION } = options;
            try {
                const item = api.getItem(cacheName);
                if (item) {
                    const root = JSON.parse(item);
                    if (root && root.data[key] && root.data[key].version === version) { // eslint-disable-line
                        if (root.data[key].expires < Date.now()) {
                            this.remove(key);
                            return undefined;
                        }
                        return root.data[key].data;
                    }
                }
                return undefined;
            }
            catch (error) {
                return undefined;
            }
        },
        set(key, data, options = { version: VERSION, time: '7d' }) {
            const { version = VERSION, time = '7d' } = options;
            try {
                const item = api.getItem(cacheName);
                if (item && item.length > 1024 * 1024 * 3)
                    { this.clear(); }
                const root = item ? JSON.parse(item) : { version, data: {} };
                root.data[key] = { version, data, expires: +(new Date()) + getTime(time) };
                api.setItem(cacheName, JSON.stringify(root));
                return true;
            }
            catch (error) {
                if (error.name === 'QuotaExceededError') {
                    api.removeItem(cacheName);
                    return this.set(key, data, options);
                }
                return false;
            }
        },
        clear() {
            try {
                api.removeItem(cacheName);
                return true;
            }
            catch (error) {
                return false;
            }
        },
        remove(key) {
            try {
                const item = api.getItem(cacheName);
                if (item) {
                    const root = JSON.parse(item);
                    delete root.data[key];
                    return true;
                }
                return false;
            }
            catch (error) {
                return false;
            }
        },
    };
}
class MenoryStorage {
    constructor() {
        this.cache = {};
    }
    getItem(key) {
        if (key in this.cache) {
            return this.cache[key];
        }
        return null;
    }
    setItem(key, data) {
        this.cache[key] = JSON.stringify(data);
    }
    removeItem(key) {
        delete this.cache[key];
    }
}
export const localCache = createCache(localStorage);
export const sessionCache = createCache(sessionStorage);
export const menoryCache = createCache(new MenoryStorage());
