import { createWrapper } from '@vue/test-utils';

import request from '../../request';
import { questionnaire } from '../..';


describe('Test questionnaire component', () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });

    it('Run questionnaire() will render then questionnaire', async() => {
        request.get.mockImplementation(() => Promise.resolve({ data: [{}] }));

        const { vm } = questionnaire('2');

        await global.delay(100);

        const wrapper = createWrapper(vm);

        expect(wrapper.findAll('.gd-questionnaire').length).toEqual(1);

        vm.onCancel();
    });

    it('Run questionnaire() will not render then questionnaire', async() => {
        request.get.mockImplementation(() => Promise.resolve({ data: [] }));

        const { vm } = questionnaire('2');
        await global.delay(100);

        expect(vm.$el).toBeUndefined();
    });


    it('Submit data successfully', async() => {
        request.get.mockImplementation(() => Promise.resolve({ data: [{}] }));
        const { vm } = questionnaire('2');

        await global.delay(100);
        const wrapper = createWrapper(vm);

        vm.vm.rate = 3;
        vm.vm.nps = 5;

        const fn = jest.fn(() => Promise.resolve());
        vm.vm.submit = fn;

        await global.delay(100);

        wrapper.find({ name: 'gd-button' }).trigger('click');

        expect(fn).toHaveBeenCalled();

        await global.delay(100);

        expect(document.querySelectorAll('.gd-confirm .gd-icon--success').length).toEqual(1);

        expect(document.querySelectorAll('.gd-questionnaire').length).toEqual(0);

        await global.delay(1000);

        expect(wrapper.element).toMatchSnapshot();
    });

    it('Failed to submit data', async() => {
        request.get.mockImplementation(() => Promise.resolve({ data: [{}] }));
        const { vm } = questionnaire('2');

        await global.delay(100);
        const wrapper = createWrapper(vm);

        vm.vm.rate = 3;
        vm.vm.nps = 5;

        let fn = jest.fn(() => Promise.reject(new Error()));
        vm.vm.submit = fn;

        await global.delay(100);

        wrapper.find({ name: 'gd-button' }).trigger('click');

        expect(fn).toHaveBeenCalled();

        await global.delay(100);

        expect(document.querySelectorAll('.gd-confirm .gd-icon--error').length).toEqual(1);

        expect(document.querySelectorAll('.gd-questionnaire').length).toEqual(0);

        fn = jest.fn(() => Promise.resolve());
        vm.vm.submit = fn;

        document.querySelector('.gd-button').click();

        await global.delay(100);

        expect(document.querySelectorAll('.gd-confirm .gd-icon--error').length).toEqual(0);
        expect(document.querySelectorAll('.gd-confirm .gd-icon--success').length).toEqual(1);

        expect(wrapper.element).toMatchSnapshot();
    });
});
