/*
 * @Author: lijianzhang
 * @Date: 2018-11-02 14:12:15
 * @Last Modified by: 天吾
 * @Last Modified time: 2019-05-01 23:17:27
 */
import merge from 'lodash/merge';
import { REQUEST_URL } from '../_utils/constant';
import * as cache from '../cache';
import cloneDeep from 'lodash/cloneDeep';
import { Pagination } from './pagination';
import pathToRegexp from 'path-to-regexp';
import Qs from 'qs';
import Axios from 'axios';
export function createRequest(config) {
    const request = Axios.create(Object.assign({}, {
        paramsSerializer: params => {
            return Qs.stringify(params, {
                arrayFormat: 'repeat'
            });
        }
    }, config));
    // 支持 /user/:id 格式
    request.interceptors.request.use((config) => {
        let url = config.url || '';
        const params = config.params;
        let data = config.data || {};
        try {
            let domain = '';
            const urlMatch = url.match(/[a-zA-z]+:\/\/[^/]*/);
            if (urlMatch) {
                [domain] = urlMatch;
                url = url.slice(domain.length);
            }
            const match = pathToRegexp.parse(url);
            if (!match) return;
            url = pathToRegexp.compile(url)(Object.assign({}, params, data));
            for (const item of match) {
                if (!(item instanceof Object))
                    { continue; }
                if (item.name in params) {
                    delete params[item.name];
                }
                else if (item.name in data) {
                    delete data[item.name];
                }
                else if (params[item.name] === '') {
                    delete params[item.name];
                }
            }
            url = domain + url;
            config.url = url;
            config.params = params;
            config.data = data;
        }
        catch (e) {
            console.error(e.message);
            throw e;
        }
        return config;
    }, (error) => error);
    request.interceptors.response.use((res) => {
        if (process.env.VUE_APP_MOCK === 'true') {
            res.data = res.data.data ? res.data.data : res.data;
        }
        if (res.headers['x-pagination']) {
            try {
                res.pagination = new Pagination(JSON.parse(res.headers['x-pagination']));
            }
            catch (error) {
                throw new Error('无效的 pagination 数据');
            }
        }
        return res;
    }, (err) => {
        if (err.message === 'Network Error') {
            throw new Error('网络加载失败，请检查您的网络设置');
        }
        if (!err.response)
            { throw err; }
        if (STATUS_CODES[err.response.status]) {
            throw new Error(STATUS_CODES[err.response.status]);
        }
        else {
            throw new Error(err.response.data.message || err.response.data);
        }
    });
    function getRequestId(baseURL, url, params = {}) {
        return baseURL + (url || '') + '?' + Qs.stringify(Object.assign({}, params));
    }
    /** ********* 缓存&重复请求 ********************/
    const defalutAdopter = request.defaults.adapter;
    const requestCache = {};
    request.defaults.adapter = (config) => {
        if (config.method === 'get') {
            const key = getRequestId(config.baseURL || '', config.url, config.params);
            const cacheConfig = typeof config.cache === 'object' ? config.cache : { type: 'sessionCache' };
            const storageName = cacheConfig.type || 'sessionCache';
            if (requestCache[key]) {
                return requestCache[key];
            }
            else if (config.cache) {
                const data = cache[storageName].get(key);
                if (data) {
                    // 如果缓存配置了重新校验， 获取缓存后，需要再请求次数据更新缓存
                    if (cacheConfig.revalidate) {
                        // 目前的config.url 是已经加上 baseURL 的地址 baseURL 需要设置为空，否者再次请求的时候 有会加上 baseURL
                        config.baseURL = '';
                        request.request(Object.assign({}, config, { cache: false })).then((res) => {
                            cache[storageName].set(key, Object.assign({}, { isCache: true }, res, {
                                time: cacheConfig.time || '10m',
                                version: cacheConfig.version,
                            }));
                        }).catch(() => {
                            //
                        });
                    }
                    return Promise.resolve(data);
                }
            }
            const promise = defalutAdopter(config).then((res) => {
                delete requestCache[key];
                if (config.cache) {
                    cache[storageName].set(key, Object.assign({}, { isCache: true }, res), {
                        time: cacheConfig.time || '10m',
                        version: cacheConfig.version,
                    });
                }
                return res;
            }).catch((error) => {
                delete requestCache[key];
                throw error;
            });
            requestCache[key] = promise;
            return promise;
        }
        return defalutAdopter(config);
    };
    return request;
}
const CancelToken = Axios.CancelToken;
const baseURL = REQUEST_URL;
// const baseURL = 'http://www.dev.gaoding.com';
const request = createRequest({
    baseURL,
    withCredentials: true,
    timeout: 30 * 1000,
});
const STATUS_CODES = {
    '401': 'User not authenticated',
};
request.interceptors.response.use(res => {
    if (res.headers['x-pagination']) {
        try {
            res.pagination = JSON.parse(res.headers['x-pagination']);
        }
        catch (error) {
            throw new Error('无效的 pagination 数据');
        }
    }
    return res;
}, err => {
    if (err.message === 'Network Error')
        { throw new Error('网络加载失败，请检查您的网络设置'); }
    if (!err.response)
        { throw err; }
    if (STATUS_CODES[err.response.status]) {
        throw new Error(STATUS_CODES[err.response.status]);
    }
    else {
        throw new Error(err.response.data.message || err.response.data);
    }
});
const post = request.post.bind(request);
request.post = (url, data = {}, config = {}) => {
    let cancel;
    const cancelToken = new CancelToken(function executor(c) {
        // An executor function receives a cancel function as a parameter
        cancel = c;
    });
    const promise = post(url, data, merge(config, { cancelToken }));
    promise.cancel = cancel;
    return promise;
};
request.setBaseURL = function(url) {
    request.defaults.baseURL = url;
};

export default request;
/** ********* 缓存&重复请求 END ********************/
